#pragma once

#include <map>
#include <set>
#include <string>
#include <tuple>
#include <vector>

#include <krpc/decoder.hpp>
#include <krpc/encoder.hpp>
#include <krpc/error.hpp>
#include <krpc/event.hpp>
#include <krpc/object.hpp>
#include <krpc/service.hpp>
#include <krpc/stream.hpp>

namespace krpc {
namespace services {

class KRPC : public Service {
 public:
  explicit KRPC(Client* client);

  // Class forward declarations
  class Expression;

  /**
   * A method was invoked where at least one of the passed arguments does not
   * meet the parameter specification of the method.
   */
  class ArgumentException : public ::krpc::RPCError {
   public:
    inline explicit ArgumentException(const std::string& msg) : ::krpc::RPCError(msg) {}
  };

  /**
   * A null reference was passed to a method that does not accept it as a valid argument.
   */
  class ArgumentNullException : public ::krpc::RPCError {
   public:
    inline explicit ArgumentNullException(const std::string& msg) : ::krpc::RPCError(msg) {}
  };

  /**
   * The value of an argument is outside the allowable range of values as defined by the invoked method.
   */
  class ArgumentOutOfRangeException : public ::krpc::RPCError {
   public:
    inline explicit ArgumentOutOfRangeException(const std::string& msg) : ::krpc::RPCError(msg) {}
  };

  /**
   * A method call was made to a method that is invalid
   * given the current state of the object.
   */
  class InvalidOperationException : public ::krpc::RPCError {
   public:
    inline explicit InvalidOperationException(const std::string& msg) : ::krpc::RPCError(msg) {}
  };

  /**
   * The game scene. See KRPC::current_game_scene.
   */
  enum struct GameScene {
    /**
     * The game scene showing the Kerbal Space Center buildings.
     */
    space_center = 0,
    /**
     * The game scene showing a vessel in flight (or on the launchpad/runway).
     */
    flight = 1,
    /**
     * The tracking station.
     */
    tracking_station = 2,
    /**
     * The Vehicle Assembly Building.
     */
    editor_vab = 3,
    /**
     * The Space Plane Hangar.
     */
    editor_sph = 4
  };

  /**
   * Create an event from a server side expression.
   */
  ::krpc::Event add_event(KRPC::Expression expression);

  /**
   * Add a streaming request and return its identifier.
   */
  krpc::schema::Stream add_stream(krpc::schema::ProcedureCall call, bool start);

  /**
   * Returns the identifier for the current client.
   */
  std::string get_client_id();

  /**
   * Returns the name of the current client.
   * This is an empty string if the client has no name.
   */
  std::string get_client_name();

  /**
   * Returns information on all services, procedures, classes, properties etc. provided by the server.
   * Can be used by client libraries to automatically create functionality such as stubs.
   */
  krpc::schema::Services get_services();

  /**
   * Returns some information about the server, such as the version.
   */
  krpc::schema::Status get_status();

  /**
   * Remove a streaming request.
   */
  void remove_stream(google::protobuf::uint64 id);

  /**
   * Start a previously added streaming request.
   */
  void start_stream(google::protobuf::uint64 id);

  /**
   * Whether the game is paused.
   */
  bool paused();

  /**
   * Whether the game is paused.
   */
  void set_paused(bool value);

  /**
   * A list of RPC clients that are currently connected to the server.
   * Each entry in the list is a clients identifier, name and address.
   */
  std::vector<std::tuple<std::string, std::string, std::string> > clients();

  /**
   * Get the current game scene.
   */
  KRPC::GameScene current_game_scene();

  ::krpc::Stream<::krpc::Event> add_event_stream(KRPC::Expression expression);

  ::krpc::Stream<krpc::schema::Stream> add_stream_stream(krpc::schema::ProcedureCall call, bool start);

  ::krpc::Stream<std::string> get_client_id_stream();

  ::krpc::Stream<std::string> get_client_name_stream();

  ::krpc::Stream<krpc::schema::Services> get_services_stream();

  ::krpc::Stream<krpc::schema::Status> get_status_stream();

  ::krpc::Stream<bool> paused_stream();

  ::krpc::Stream<std::vector<std::tuple<std::string, std::string, std::string> >> clients_stream();

  ::krpc::Stream<KRPC::GameScene> current_game_scene_stream();

  ::krpc::schema::ProcedureCall add_event_call(KRPC::Expression expression);

  ::krpc::schema::ProcedureCall add_stream_call(krpc::schema::ProcedureCall call, bool start);

  ::krpc::schema::ProcedureCall get_client_id_call();

  ::krpc::schema::ProcedureCall get_client_name_call();

  ::krpc::schema::ProcedureCall get_services_call();

  ::krpc::schema::ProcedureCall get_status_call();

  ::krpc::schema::ProcedureCall remove_stream_call(google::protobuf::uint64 id);

  ::krpc::schema::ProcedureCall start_stream_call(google::protobuf::uint64 id);

  ::krpc::schema::ProcedureCall paused_call();

  ::krpc::schema::ProcedureCall set_paused_call(bool value);

  ::krpc::schema::ProcedureCall clients_call();

  ::krpc::schema::ProcedureCall current_game_scene_call();

  /**
   * A server side expression.
   */
  class Expression : public krpc::Object<Expression> {
   public:
    explicit Expression(Client* client = nullptr, google::protobuf::uint64 id = 0);

    /**
     * Numerical addition.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression add(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Boolean and operator.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression and_(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * An RPC call.
     * @param call
     */
    static KRPC::Expression call(Client& client, krpc::schema::ProcedureCall call);

    /**
     * A constant value of type double.
     * @param value
     */
    static KRPC::Expression constant_double(Client& client, double value);

    /**
     * A constant value of type float.
     * @param value
     */
    static KRPC::Expression constant_float(Client& client, float value);

    /**
     * A constant value of type int.
     * @param value
     */
    static KRPC::Expression constant_int(Client& client, google::protobuf::int32 value);

    /**
     * A constant value of type string.
     * @param value
     */
    static KRPC::Expression constant_string(Client& client, std::string value);

    /**
     * Numerical division.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression divide(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Equality comparison.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression equal(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Boolean exclusive-or operator.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression exclusive_or(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Greater than numerical comparison.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression greater_than(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Greater than or equal numerical comparison.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression greater_than_or_equal(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Bitwise left shift.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression left_shift(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Less than numerical comparison.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression less_than(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Less than or equal numerical comparison.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression less_than_or_equal(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Numerical modulo operator.
     * @param arg0
     * @param arg1
     * @return The remainder of arg0 divided by arg1
     */
    static KRPC::Expression modulo(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Numerical multiplication.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression multiply(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Boolean negation operator.
     * @param arg
     */
    static KRPC::Expression not_(Client& client, KRPC::Expression arg);

    /**
     * Inequality comparison.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression not_equal(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Boolean or operator.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression or_(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Numerical power operator.
     * @param arg0
     * @param arg1
     * @return arg0 raised to the power of arg1
     */
    static KRPC::Expression power(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Bitwise right shift.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression right_shift(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Numerical subtraction.
     * @param arg0
     * @param arg1
     */
    static KRPC::Expression subtract(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    /**
     * Convert to a double type.
     * @param arg
     */
    static KRPC::Expression to_double(Client& client, KRPC::Expression arg);

    /**
     * Convert to a float type.
     * @param arg
     */
    static KRPC::Expression to_float(Client& client, KRPC::Expression arg);

    /**
     * Convert to an int type.
     * @param arg
     */
    static KRPC::Expression to_int(Client& client, KRPC::Expression arg);

    static ::krpc::Stream<KRPC::Expression> add_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> and__stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> call_stream(Client& client, krpc::schema::ProcedureCall call);

    static ::krpc::Stream<KRPC::Expression> constant_double_stream(Client& client, double value);

    static ::krpc::Stream<KRPC::Expression> constant_float_stream(Client& client, float value);

    static ::krpc::Stream<KRPC::Expression> constant_int_stream(Client& client, google::protobuf::int32 value);

    static ::krpc::Stream<KRPC::Expression> constant_string_stream(Client& client, std::string value);

    static ::krpc::Stream<KRPC::Expression> divide_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> equal_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> exclusive_or_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> greater_than_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> greater_than_or_equal_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> left_shift_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> less_than_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> less_than_or_equal_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> modulo_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> multiply_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> not__stream(Client& client, KRPC::Expression arg);

    static ::krpc::Stream<KRPC::Expression> not_equal_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> or__stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> power_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> right_shift_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> subtract_stream(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::Stream<KRPC::Expression> to_double_stream(Client& client, KRPC::Expression arg);

    static ::krpc::Stream<KRPC::Expression> to_float_stream(Client& client, KRPC::Expression arg);

    static ::krpc::Stream<KRPC::Expression> to_int_stream(Client& client, KRPC::Expression arg);

    static ::krpc::schema::ProcedureCall add_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall and__call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall call_call(Client& client, krpc::schema::ProcedureCall call);

    static ::krpc::schema::ProcedureCall constant_double_call(Client& client, double value);

    static ::krpc::schema::ProcedureCall constant_float_call(Client& client, float value);

    static ::krpc::schema::ProcedureCall constant_int_call(Client& client, google::protobuf::int32 value);

    static ::krpc::schema::ProcedureCall constant_string_call(Client& client, std::string value);

    static ::krpc::schema::ProcedureCall divide_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall equal_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall exclusive_or_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall greater_than_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall greater_than_or_equal_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall left_shift_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall less_than_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall less_than_or_equal_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall modulo_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall multiply_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall not__call(Client& client, KRPC::Expression arg);

    static ::krpc::schema::ProcedureCall not_equal_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall or__call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall power_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall right_shift_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall subtract_call(Client& client, KRPC::Expression arg0, KRPC::Expression arg1);

    static ::krpc::schema::ProcedureCall to_double_call(Client& client, KRPC::Expression arg);

    static ::krpc::schema::ProcedureCall to_float_call(Client& client, KRPC::Expression arg);

    static ::krpc::schema::ProcedureCall to_int_call(Client& client, KRPC::Expression arg);
  };
};

}  // namespace services

namespace encoder {

  inline std::string encode(const services::KRPC::GameScene& value) {
    return krpc::encoder::encode(static_cast<google::protobuf::int32>(value));
  }

}  // namespace encoder

namespace decoder {

  inline void decode(services::KRPC::GameScene& value, const std::string& data, Client* client) {
    google::protobuf::int32 x;
    decode(x, data, client);
    value = static_cast<services::KRPC::GameScene>(x);
  }

}  // namespace decoder

namespace services {

inline KRPC::KRPC(Client* client):
  Service(client) {
  client->add_exception_thrower(
    "KRPC", "ArgumentException",
    [] (const std::string& msg) { throw ArgumentException(msg); });
  client->add_exception_thrower(
    "KRPC", "ArgumentNullException",
    [] (const std::string& msg) { throw ArgumentNullException(msg); });
  client->add_exception_thrower(
    "KRPC", "ArgumentOutOfRangeException",
    [] (const std::string& msg) { throw ArgumentOutOfRangeException(msg); });
  client->add_exception_thrower(
    "KRPC", "InvalidOperationException",
    [] (const std::string& msg) { throw InvalidOperationException(msg); });
}

inline ::krpc::Event KRPC::add_event(KRPC::Expression expression) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(expression));
  std::string _data = this->_client->invoke("KRPC", "AddEvent", _args);
  ::krpc::Event _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline krpc::schema::Stream KRPC::add_stream(krpc::schema::ProcedureCall call, bool start = true) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(call));
  _args.push_back(encoder::encode(start));
  std::string _data = this->_client->invoke("KRPC", "AddStream", _args);
  krpc::schema::Stream _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline std::string KRPC::get_client_id() {
  std::string _data = this->_client->invoke("KRPC", "GetClientID");
  std::string _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline std::string KRPC::get_client_name() {
  std::string _data = this->_client->invoke("KRPC", "GetClientName");
  std::string _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline krpc::schema::Services KRPC::get_services() {
  std::string _data = this->_client->invoke("KRPC", "GetServices");
  krpc::schema::Services _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline krpc::schema::Status KRPC::get_status() {
  std::string _data = this->_client->invoke("KRPC", "GetStatus");
  krpc::schema::Status _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline void KRPC::remove_stream(google::protobuf::uint64 id) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(id));
  this->_client->invoke("KRPC", "RemoveStream", _args);
}

inline void KRPC::start_stream(google::protobuf::uint64 id) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(id));
  this->_client->invoke("KRPC", "StartStream", _args);
}

inline bool KRPC::paused() {
  std::string _data = this->_client->invoke("KRPC", "get_Paused");
  bool _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline void KRPC::set_paused(bool value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  this->_client->invoke("KRPC", "set_Paused", _args);
}

inline std::vector<std::tuple<std::string, std::string, std::string> > KRPC::clients() {
  std::string _data = this->_client->invoke("KRPC", "get_Clients");
  std::vector<std::tuple<std::string, std::string, std::string> > _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline KRPC::GameScene KRPC::current_game_scene() {
  std::string _data = this->_client->invoke("KRPC", "get_CurrentGameScene");
  KRPC::GameScene _result;
  decoder::decode(_result, _data, this->_client);
  return _result;
}

inline ::krpc::Stream<::krpc::Event> KRPC::add_event_stream(KRPC::Expression expression) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(expression));
  return ::krpc::Stream<::krpc::Event>(this->_client, this->_client->build_call("KRPC", "AddEvent", _args));
}

inline ::krpc::Stream<krpc::schema::Stream> KRPC::add_stream_stream(krpc::schema::ProcedureCall call, bool start = true) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(call));
  _args.push_back(encoder::encode(start));
  return ::krpc::Stream<krpc::schema::Stream>(this->_client, this->_client->build_call("KRPC", "AddStream", _args));
}

inline ::krpc::Stream<std::string> KRPC::get_client_id_stream() {
  return ::krpc::Stream<std::string>(this->_client, this->_client->build_call("KRPC", "GetClientID"));
}

inline ::krpc::Stream<std::string> KRPC::get_client_name_stream() {
  return ::krpc::Stream<std::string>(this->_client, this->_client->build_call("KRPC", "GetClientName"));
}

inline ::krpc::Stream<krpc::schema::Services> KRPC::get_services_stream() {
  return ::krpc::Stream<krpc::schema::Services>(this->_client, this->_client->build_call("KRPC", "GetServices"));
}

inline ::krpc::Stream<krpc::schema::Status> KRPC::get_status_stream() {
  return ::krpc::Stream<krpc::schema::Status>(this->_client, this->_client->build_call("KRPC", "GetStatus"));
}

inline ::krpc::Stream<bool> KRPC::paused_stream() {
  return ::krpc::Stream<bool>(this->_client, this->_client->build_call("KRPC", "get_Paused"));
}

inline ::krpc::Stream<std::vector<std::tuple<std::string, std::string, std::string> >> KRPC::clients_stream() {
  return ::krpc::Stream<std::vector<std::tuple<std::string, std::string, std::string> >>(this->_client, this->_client->build_call("KRPC", "get_Clients"));
}

inline ::krpc::Stream<KRPC::GameScene> KRPC::current_game_scene_stream() {
  return ::krpc::Stream<KRPC::GameScene>(this->_client, this->_client->build_call("KRPC", "get_CurrentGameScene"));
}

inline ::krpc::schema::ProcedureCall KRPC::add_event_call(KRPC::Expression expression) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(expression));
  return this->_client->build_call("KRPC", "AddEvent", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::add_stream_call(krpc::schema::ProcedureCall call, bool start = true) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(call));
  _args.push_back(encoder::encode(start));
  return this->_client->build_call("KRPC", "AddStream", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::get_client_id_call() {
  return this->_client->build_call("KRPC", "GetClientID");
}

inline ::krpc::schema::ProcedureCall KRPC::get_client_name_call() {
  return this->_client->build_call("KRPC", "GetClientName");
}

inline ::krpc::schema::ProcedureCall KRPC::get_services_call() {
  return this->_client->build_call("KRPC", "GetServices");
}

inline ::krpc::schema::ProcedureCall KRPC::get_status_call() {
  return this->_client->build_call("KRPC", "GetStatus");
}

inline ::krpc::schema::ProcedureCall KRPC::remove_stream_call(google::protobuf::uint64 id) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(id));
  return this->_client->build_call("KRPC", "RemoveStream", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::start_stream_call(google::protobuf::uint64 id) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(id));
  return this->_client->build_call("KRPC", "StartStream", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::paused_call() {
  return this->_client->build_call("KRPC", "get_Paused");
}

inline ::krpc::schema::ProcedureCall KRPC::set_paused_call(bool value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return this->_client->build_call("KRPC", "set_Paused", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::clients_call() {
  return this->_client->build_call("KRPC", "get_Clients");
}

inline ::krpc::schema::ProcedureCall KRPC::current_game_scene_call() {
  return this->_client->build_call("KRPC", "get_CurrentGameScene");
}

inline KRPC::Expression::Expression(Client* client, google::protobuf::uint64 id):
  Object(client, "KRPC::Expression", id) {}

inline KRPC::Expression KRPC::Expression::add(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Add", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::and_(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_And", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::call(Client& _client, krpc::schema::ProcedureCall call) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(call));
  std::string _data = _client.invoke("KRPC", "Expression_static_Call", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::constant_double(Client& _client, double value) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  std::string _data = _client.invoke("KRPC", "Expression_static_ConstantDouble", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::constant_float(Client& _client, float value) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  std::string _data = _client.invoke("KRPC", "Expression_static_ConstantFloat", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::constant_int(Client& _client, google::protobuf::int32 value) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  std::string _data = _client.invoke("KRPC", "Expression_static_ConstantInt", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::constant_string(Client& _client, std::string value) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  std::string _data = _client.invoke("KRPC", "Expression_static_ConstantString", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::divide(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Divide", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::equal(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Equal", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::exclusive_or(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_ExclusiveOr", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::greater_than(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_GreaterThan", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::greater_than_or_equal(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_GreaterThanOrEqual", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::left_shift(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_LeftShift", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::less_than(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_LessThan", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::less_than_or_equal(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_LessThanOrEqual", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::modulo(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Modulo", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::multiply(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Multiply", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::not_(Client& _client, KRPC::Expression arg) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  std::string _data = _client.invoke("KRPC", "Expression_static_Not", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::not_equal(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_NotEqual", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::or_(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Or", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::power(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Power", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::right_shift(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_RightShift", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::subtract(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  std::string _data = _client.invoke("KRPC", "Expression_static_Subtract", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::to_double(Client& _client, KRPC::Expression arg) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  std::string _data = _client.invoke("KRPC", "Expression_static_ToDouble", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::to_float(Client& _client, KRPC::Expression arg) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  std::string _data = _client.invoke("KRPC", "Expression_static_ToFloat", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline KRPC::Expression KRPC::Expression::to_int(Client& _client, KRPC::Expression arg) {
    std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  std::string _data = _client.invoke("KRPC", "Expression_static_ToInt", _args);
    KRPC::Expression _result;
  decoder::decode(_result, _data, &_client);
  return _result;
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::add_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Add", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::and__stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_And", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::call_stream(Client& _client, krpc::schema::ProcedureCall call) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(call));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Call", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::constant_double_stream(Client& _client, double value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ConstantDouble", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::constant_float_stream(Client& _client, float value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ConstantFloat", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::constant_int_stream(Client& _client, google::protobuf::int32 value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ConstantInt", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::constant_string_stream(Client& _client, std::string value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ConstantString", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::divide_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Divide", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::equal_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Equal", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::exclusive_or_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ExclusiveOr", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::greater_than_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_GreaterThan", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::greater_than_or_equal_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_GreaterThanOrEqual", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::left_shift_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_LeftShift", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::less_than_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_LessThan", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::less_than_or_equal_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_LessThanOrEqual", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::modulo_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Modulo", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::multiply_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Multiply", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::not__stream(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Not", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::not_equal_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_NotEqual", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::or__stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Or", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::power_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Power", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::right_shift_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_RightShift", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::subtract_stream(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_Subtract", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::to_double_stream(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ToDouble", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::to_float_stream(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ToFloat", _args));
}

inline ::krpc::Stream<KRPC::Expression> KRPC::Expression::to_int_stream(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return ::krpc::Stream<KRPC::Expression>(&_client, _client.build_call("KRPC", "Expression_static_ToInt", _args));
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::add_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Add", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::and__call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_And", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::call_call(Client& _client, krpc::schema::ProcedureCall call) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(call));
  return _client.build_call("KRPC", "Expression_static_Call", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::constant_double_call(Client& _client, double value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return _client.build_call("KRPC", "Expression_static_ConstantDouble", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::constant_float_call(Client& _client, float value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return _client.build_call("KRPC", "Expression_static_ConstantFloat", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::constant_int_call(Client& _client, google::protobuf::int32 value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return _client.build_call("KRPC", "Expression_static_ConstantInt", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::constant_string_call(Client& _client, std::string value) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(value));
  return _client.build_call("KRPC", "Expression_static_ConstantString", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::divide_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Divide", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::equal_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Equal", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::exclusive_or_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_ExclusiveOr", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::greater_than_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_GreaterThan", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::greater_than_or_equal_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_GreaterThanOrEqual", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::left_shift_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_LeftShift", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::less_than_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_LessThan", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::less_than_or_equal_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_LessThanOrEqual", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::modulo_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Modulo", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::multiply_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Multiply", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::not__call(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return _client.build_call("KRPC", "Expression_static_Not", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::not_equal_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_NotEqual", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::or__call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Or", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::power_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Power", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::right_shift_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_RightShift", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::subtract_call(Client& _client, KRPC::Expression arg0, KRPC::Expression arg1) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg0));
  _args.push_back(encoder::encode(arg1));
  return _client.build_call("KRPC", "Expression_static_Subtract", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::to_double_call(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return _client.build_call("KRPC", "Expression_static_ToDouble", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::to_float_call(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return _client.build_call("KRPC", "Expression_static_ToFloat", _args);
}

inline ::krpc::schema::ProcedureCall KRPC::Expression::to_int_call(Client& _client, KRPC::Expression arg) {
  std::vector<std::string> _args;
  _args.push_back(encoder::encode(arg));
  return _client.build_call("KRPC", "Expression_static_ToInt", _args);
}
}  // namespace services

}  // namespace krpc
